/******************************************************
 * Title: max11618_adc_model.v 
 * Author: Richard Jolly                           
 * Creation Date: 2/23/2015                           
 * Description: This Module is models the MAX11618 4 Channel
 * ADC. It is a SPI Slave device that has a speed of 300KSa/sec
 *
 * Copyright Earth People Technology, Inc. 2015
 *
******************************************************/
`include "../src/define.v"

`timescale 1 ns / 10 ps

module max11618_adc_model
  (
	input wire                  CLK,
	input wire                  RST_N,

    // ADC Interface	
	input wire                  ADC_MOSI,
	input wire                  ADC_CLK,
	output wire                 ADC_MISO,
	input wire                  ADC_CS,
	input wire                  ADC_CNVST,	
	output wire                 ADC_EOC

   );


   //----------------------------------------------
   // Parameter Declarations
   //----------------------------------------------
   parameter 	IDLE                        = 0,
                READ_MASTER                 = 1,
                ASSERT_TXE                  = 2,
                CHECK_ADC_REGISTER          = 3,
				LATCH_ADC_REGISTER          = 4,
				WAIT_FOR_EOC                = 5,
                ASSERT_ADC_CONTROL          = 6,
                READ_ADC                    = 7,
                ASSERT_USB_RXF              = 8,
                WAIT_FOR_USB_RD_N           = 9,
				WAIT_DATA_BYTE_COMPLETE     = 10,
				CHECK_DATA_MSB              = 11,
                CHECK_DATA_XMIT_COMPLETE    = 12;
   
   parameter 	ADC_EOC_DELAY_COUNT         = 16'h7f;
   //------------------------------------------------
   // Wire and Register Declarations                
   //------------------------------------------------
   	reg [12:0]                   state, next;


	//SPI Slave Registers
   	reg                         spi_slave_xmt_en;
   	reg                         spi_slave_rcv_rdy;
   	wire   [7:0]                spi_slave_data;     
   	wire                        spi_slave_xmt_rdy;
   	wire                        spi_slave_rcv_en;
	
	//ADC Registers
   	wire   [9:0]                adc_data_out;
   	reg                         adc_cs;
   	reg                         adc_read;
	wire                        adc_busy;
	wire                        adc_conv_start;
	reg                         adc_end_of_conv;
	
	//Data Transfer registers
   	reg   [9:0]                 transmit_data_out;
	reg                         adc_data_xmit_msb;
	reg [7:0]                   adc_internal_register;
	
    // ADC EOC Registers
	reg                         adc_eoc_reg;
   	reg	[15:0]                  adc_eoc_counter;
	
	//ADC Internal Registers
   	reg	[7:0]                   adc_conversion_register;
   	reg	[7:0]                   adc_setup_register;	
   	reg	[7:0]                   adc_averaging_register;
   	reg	[7:0]                   adc_reset_register;	
	
	//Number of Channels
	reg  [2:0]                  number_of_scan_channels;
	reg                         num_chans_decrement_reg;
	

    //TEST Output
	reg	[3:0]                   SPI_STATE;

      
`ifdef SIM
   	reg [8*30:1] state_name;
`endif
   
   //------------------------------------------------
   // Signal Assignments  
   //------------------------------------------------
   
   //SPI data register, bi-directional
   assign   spi_slave_data = (state[WAIT_FOR_USB_RD_N] & adc_data_xmit_msb) ? {6'h0,transmit_data_out[9:8]} : ((state[WAIT_FOR_USB_RD_N] & !adc_data_xmit_msb) ? transmit_data_out[7:0] : 10'hzzzz);
   //assign   ADC_EOC = state[ASSERT_ADC_CONTROL] ? 1'b1 : 1'b0;
   
   assign  adc_conv_start = ~ADC_CNVST;
   assign  ADC_EOC = ~adc_end_of_conv;
   
   //-----------------------------------------------
   // ADC Cycle: Set Number of channels to scan 
   //-----------------------------------------------

   always @( negedge CLK or negedge RST_N )
   begin
      if ( !RST_N )
      begin
         number_of_scan_channels <= 3'h0;
		 num_chans_decrement_reg <= 1'b0;
      end
      else
      begin
         if(state[ASSERT_ADC_CONTROL])
         begin
		    case(adc_conversion_register)
			8'h00:
               number_of_scan_channels <= 3'h1;
			8'h08:
               number_of_scan_channels <= 3'h2;
			8'h10:
               number_of_scan_channels <= 3'h3;
			8'h18:
               number_of_scan_channels <= 3'h4;
			default:
               number_of_scan_channels <= 3'h1;
			endcase
         end
         else if(state[WAIT_FOR_USB_RD_N] & !adc_data_xmit_msb & !num_chans_decrement_reg)
         begin
            number_of_scan_channels <= number_of_scan_channels - 1'd1;
			num_chans_decrement_reg <= 1'b1;
         end
		 else if(state[WAIT_DATA_BYTE_COMPLETE] & num_chans_decrement_reg)
         begin
			num_chans_decrement_reg <= 1'b0;
         end
      end
   end

   //-----------------------------------------------
   // Assert the ADC_EOC signal to signal the ADC
   // has completed the conversion and data is ready 
   // for transmission.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  		adc_end_of_conv <= 1'b0;
			adc_eoc_reg <= 1'b0;
			adc_eoc_counter <= 0;
	end
	else 
	  begin
	     if (state[ASSERT_ADC_CONTROL] & !adc_eoc_reg)
		 begin
	  		 adc_end_of_conv <= 1'b1;
			 adc_eoc_reg <= 1'b1;
		 end
	     else if (adc_eoc_reg)
		 begin
		     if(adc_eoc_counter < ADC_EOC_DELAY_COUNT)
	  		    adc_eoc_counter <= adc_eoc_counter + 1'd1;
			 else 
			 begin
	  		    adc_end_of_conv <= 1'b0;
			    adc_eoc_reg <= 1'b0;
			    adc_eoc_counter <= 0;
			 end
		 end
	  end
	end


   //-----------------------------------------------
   // Assert teh USB_RXF signal to start the transmit
   // of ADC data over SPI.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  		spi_slave_xmt_en <= 1'b1;
	end
	else 
	  begin
	     if (state[ASSERT_USB_RXF])
		 begin
	  		 spi_slave_xmt_en <= 1'b0;
		 end
	     else if (state[WAIT_FOR_USB_RD_N])
		 begin
	  		 spi_slave_xmt_en <= 1'b1;
		 end
	  end
	end

   //-----------------------------------------------
   // Set the adc_data_xmit_msb to high when the initial
   // read of the ADC occurs. This will cause the state
   // machine to transmit the MSB first. The state[CHECK_DATA_XMIT_COMPLETE]
   // will then swap the polarity of the adc_data_xmit_msb
   // and send the state machine to state[ASSERT_USB_RXF]
   // to transmit the LSB.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
			 adc_data_xmit_msb <= 1'b0;
	end
	else 
	  begin
	     if (state[READ_ADC] | state[CHECK_DATA_XMIT_COMPLETE])
		 begin
			 adc_data_xmit_msb <= 1'b1;
		 end
	     if (state[CHECK_DATA_MSB])
		 begin
			 adc_data_xmit_msb <= 1'b0;
		 end
	  end
	end

   //-----------------------------------------------
   // Read the ADC Data.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  		transmit_data_out <= 0;
	end
	else 
	  begin
	     if (state[ASSERT_USB_RXF] & adc_data_xmit_msb)
		 begin
	  		 transmit_data_out <= adc_data_out;
		 end
	  end
	end

	//-----------------------------------------------
   // Assert ADC controls to read the ADC Data.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	begin
	  		adc_cs <= 1'b1;
		  	adc_read <= 1'b1;
	  		adc_averaging_register <= 0;
		  	adc_reset_register <= 0;
	end
	else 
	  begin
	     if (state[ASSERT_USB_RXF])
		 begin
		    if(adc_data_xmit_msb)
			begin
	  		   adc_cs <= 1'b0;
		  	   adc_read <= 1'b0;
			end
		 end
	     else if (state[WAIT_FOR_USB_RD_N])
		 begin
	  		adc_cs <= 1'b1;
		  	adc_read <= 1'b1;
		 end
	  end
	end

	  //-----------------------------------------------
   // Select an ADC Register and send the contents
   // to the ADC.
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
   begin
	if (!RST_N)
	begin
	  		adc_conversion_register <= 0;
		  	adc_setup_register <= 0;
	  		adc_averaging_register <= 0;
		  	adc_reset_register <= 0;
	end
	else 
	  begin
	     if (state[LATCH_ADC_REGISTER])
		 begin
	        casex(adc_internal_register[7:4])
			      4'b1xxx: //ADC_INT_REG_CONV:
                  begin
				     adc_conversion_register = {1'b0,adc_internal_register[6:0]};
				  end
			      4'b01xx: //ADC_INT_REG_SETUP:
			      begin
				     adc_setup_register = {2'b00,adc_internal_register[5:0]}; 
				  end
                  4'b001x: //ADC_INT_REG_AVERAGING:
                  begin
				     adc_averaging_register = {3'b000,adc_internal_register[4:0]}; 
                  end
                  4'b0001: //ADC_INT_REG_RESET:
                  begin
				     adc_reset_register = {4'b0000,adc_internal_register[3:0]};
                  end				  
			default:
			      begin
			       	adc_conversion_register <= 0;
		  	        adc_setup_register <= 0;
			       	adc_averaging_register <= 0;
			       	adc_reset_register <= 0;
				  end

			endcase
		 end
		end
	 end

   //-----------------------------------------------
   // Assert the USB_TXE signal to the SPI Slave Unit
   // 
   //-----------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  spi_slave_rcv_rdy <= 1'b0;
	else 
	  begin
	     if (state[ASSERT_TXE])
	       spi_slave_rcv_rdy <= 1'b1;
	     else if ( state[IDLE] ) 
	       spi_slave_rcv_rdy <= 1'b0;
	  end
     end // always @ (posedge CLK or negedge RST_N)   
 
	 
   //------------------------------------------------
   // Read the Data from SPI Slave unit
   //------------------------------------------------
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  adc_internal_register <= 0;
	else
	  begin
	     //if (state[READ_MASTER])
	     if (state[CHECK_ADC_REGISTER])
	       adc_internal_register <= spi_slave_data;
	  end
     end // always @ (posedge CLK or negedge RST_N)
	 

   //--------------------------------------------------
   // Write State Machine
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK or negedge RST_N)
     begin
	if (!RST_N)
	  begin
	     state <= 12'h000;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end


   always @ ( state or spi_slave_rcv_en or adc_busy
	      or spi_slave_rcv_rdy or adc_data_xmit_msb or spi_slave_xmt_rdy
	      or spi_slave_xmt_en or adc_conv_start or adc_cs or adc_read)

     begin
	next = 12'h000;

	if (state[IDLE])
	  begin
	     if(!spi_slave_rcv_en)
	       next[READ_MASTER] = 1'b1;
	     else if(adc_busy)
	       next[WAIT_FOR_EOC] = 1'b1;
		else
	       next[IDLE] = 1'b1;			
	  end

	if (state[READ_MASTER])
	  next[ASSERT_TXE] = 1'b1;

	if (state[ASSERT_TXE])
	  begin
		if(spi_slave_rcv_rdy)
			next[CHECK_ADC_REGISTER] = 1'b1;
	     else
			next[ASSERT_TXE] = 1'b1;
	  end

	if (state[CHECK_ADC_REGISTER])
		    next[LATCH_ADC_REGISTER] = 1'b1;

	if (state[LATCH_ADC_REGISTER])
		    next[IDLE] = 1'b1;

	if (state[WAIT_FOR_EOC])
	begin
	    if(!adc_busy)
		    next[ASSERT_ADC_CONTROL] = 1'b1;
	    else 
		    next[WAIT_FOR_EOC] = 1'b1;
	end

	if (state[ASSERT_ADC_CONTROL])
		    next[READ_ADC] = 1'b1;

	if (state[READ_ADC])
	  next[ASSERT_USB_RXF] = 1'b1;

	if (state[ASSERT_USB_RXF])
	       next[WAIT_FOR_USB_RD_N] = 1'b1;

	if (state[WAIT_FOR_USB_RD_N])
	  begin
	  	if(!spi_slave_xmt_rdy)
			next[WAIT_DATA_BYTE_COMPLETE] = 1'b1;
		else
			next[WAIT_FOR_USB_RD_N] = 1'b1;			
	  end 

	if (state[WAIT_DATA_BYTE_COMPLETE])
	begin
		if(spi_slave_xmt_rdy)
			next[CHECK_DATA_MSB] = 1'b1;			
		else 
			next[WAIT_DATA_BYTE_COMPLETE] = 1'b1;	
	end

	if (state[CHECK_DATA_MSB])
	begin
        if(adc_data_xmit_msb)
		   next[ASSERT_USB_RXF] = 1'b1;
		else 
			next[CHECK_DATA_XMIT_COMPLETE] = 1'b1;	
	end

	
	if (state[CHECK_DATA_XMIT_COMPLETE])
	begin
        if(number_of_scan_channels > 0)
		   next[ASSERT_USB_RXF] = 1'b1;
		else 
			next[IDLE] = 1'b1;	
	end
	
`ifdef SIM
	if (state == (1 << IDLE))
	  state_name = "IDLE";
	else if (state == (1 << READ_MASTER))
	  state_name = "READ_MASTER";
	else if (state == (1 << ASSERT_TXE))
	  state_name = "ASSERT_TXE";
	else if (state == (1 << CHECK_ADC_REGISTER))
	  state_name = "CHECK_ADC_REGISTER";
	else if (state == (1 << LATCH_ADC_REGISTER))
	  state_name = "LATCH_ADC_REGISTER";
	else if (state == (1 << WAIT_FOR_EOC))
	  state_name = "WAIT_FOR_EOC";
	else if (state == (1 << ASSERT_ADC_CONTROL))
	  state_name = "ASSERT_ADC_CONTROL";
	else if (state == (1 << READ_ADC))
	  state_name = "READ_ADC";
	else if (state == (1 << ASSERT_USB_RXF))
	  state_name = "ASSERT_USB_RXF";
	else if (state == (1 << WAIT_FOR_USB_RD_N))
	  state_name = "WAIT_FOR_USB_RD_N";
	else if (state == (1 << WAIT_DATA_BYTE_COMPLETE))
	  state_name = "WAIT_DATA_BYTE_COMPLETE";
	else if (state == (1 << CHECK_DATA_MSB))
	  state_name = "CHECK_DATA_MSB";
	else if (state == (1 << CHECK_DATA_XMIT_COMPLETE))
	  state_name = "CHECK_DATA_XMIT_COMPLETE";

	  
`endif
     end // always @ (...

   //----------------------------------------------
   // Instantiate the Leaf FSM Modules
   //----------------------------------------------
   spi_tiny_slave_model         SPI_TINY_SLAVE_MODEL_INST
     (
      .CLK                      (CLK), 
      .RST_N                    (RST_N),
      
      .SPI_DATA_OUT             (ADC_MISO),
      .SPI_CLK                  (ADC_CLK),
      .SPI_DATA_IN              (ADC_MOSI),
      .SPI_CS_N                 (ADC_CS),

	  .USB_RXF_N                (spi_slave_xmt_en),
      .USB_TXE_N                (spi_slave_rcv_rdy),
	  .USB_DATA                 (spi_slave_data),
      
	  .USB_RD_N			        (spi_slave_xmt_rdy),
	  .USB_WR                   (spi_slave_rcv_en),
	  .USB_SI                   ()
      );

   adc_model                    ADC_MODEL_INST
     (
      .CLK                      (CLK), 
      .RESET                    (RST_N),

      .AD_DATA                  (adc_data_out),
      .AD_BUSY                  (adc_busy),  
      .AD_REFOUT                (),

      .AD_CSL                   (adc_cs),
      .AD_RDL                   (adc_read),
      .AD_CONVST                (adc_conv_start),
	  
	  .ADC_CONVERSION_REG       (adc_conversion_register),
	  .ADC_SETUP_REG            (adc_setup_register),
	  .ADC_AVERAGING_REG        (adc_averaging_register),
	  .ADC_RESET_REG            (adc_reset_register)

      );

endmodule // audio_SPI_fsm

   